/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright held by original author
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "asymSlipGenNewtoniansemiFvPatchField.H"
#include "addToRunTimeSelectionTable.H"
#include "fvPatchFieldMapper.H"
#include "volFields.H"
#include "surfaceFields.H"
#include "symmTransformField.H"
#include "singlePhaseTransportModel.H"
#include "turbulenceModel.H"
#include "fieldTypes.H"
#include "incompressible/turbulenceModel/turbulenceModel.H"
#include "RASModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

asymSlipGenNewtoniansemiFvPatchVectorField::asymSlipGenNewtoniansemiFvPatchVectorField
(
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF
)
:
    fixedValueFvPatchField<vector>(p, iF),
    rho_(0),
    ka2_(0),
    ka1_(0),
    relaxationFactor_(0)
{}


asymSlipGenNewtoniansemiFvPatchVectorField::asymSlipGenNewtoniansemiFvPatchVectorField
(
    const asymSlipGenNewtoniansemiFvPatchVectorField& ptf,
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    fixedValueFvPatchField<vector>(ptf, p, iF, mapper),
    rho_(ptf.rho_),
    ka2_(ptf.ka2_),
    ka1_(ptf.ka1_),
    relaxationFactor_(ptf.relaxationFactor_)
{}


asymSlipGenNewtoniansemiFvPatchVectorField::asymSlipGenNewtoniansemiFvPatchVectorField
(
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF,
    const dictionary& dict
)
:
    fixedValueFvPatchField<vector>(p, iF),
    rho_(readScalar(dict.lookup("rho"))),
    ka2_(readScalar(dict.lookup("ka2"))),
    ka1_(readScalar(dict.lookup("ka1"))),
    relaxationFactor_(readScalar(dict.lookup("relaxationFactor")))
{
    if (dict.found("value"))
    {
        fvPatchField<vector>::operator=
        (
            vectorField("value", dict, p.size())
        );
    }
    else
    {
        // Evaluate the wall velocity
        updateCoeffs();
    }
}


asymSlipGenNewtoniansemiFvPatchVectorField::asymSlipGenNewtoniansemiFvPatchVectorField
(
    const asymSlipGenNewtoniansemiFvPatchVectorField& fcvpvf
)
:
    fixedValueFvPatchField<vector>(fcvpvf),
    rho_(fcvpvf.rho_),
    ka2_(fcvpvf.ka2_),
    ka1_(fcvpvf.ka1_),
    relaxationFactor_(fcvpvf.relaxationFactor_)
{}


asymSlipGenNewtoniansemiFvPatchVectorField::asymSlipGenNewtoniansemiFvPatchVectorField
(
    const asymSlipGenNewtoniansemiFvPatchVectorField& fcvpvf,
    const DimensionedField<vector, volMesh>& iF
)
:
    fixedValueFvPatchField<vector>(fcvpvf, iF),
    rho_(fcvpvf.rho_),
    ka2_(fcvpvf.ka2_),
    ka1_(fcvpvf.ka1_),
    relaxationFactor_(fcvpvf.relaxationFactor_)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void asymSlipGenNewtoniansemiFvPatchVectorField::updateCoeffs()
{
    if (updated())
    {
        return;
    }
    // obatin the viscosity
    const label patchI = patch().index();
    scalarField nuw = 1e-6*mag(patch().nf());  
    
    if (db().found("turbulenceModel"))
    {
	const incompressible::turbulenceModel& turbModel =
            db().lookupObject<incompressible::turbulenceModel>
            (
                "turbulenceModel"
            );

	//Info<< "\nTurbulence found\n" << endl;
	nuw = rho_*turbModel.nu()().boundaryField()[patchI];
    }
    else
    {
        nuw = mag(patch().nf());  
	//Info<< "\nTurbulence NOT found\n" << endl;
    }
    //Info<<"\n nuw_val = "<< nuw <<endl;  
   
   //-----------definition of viscosity in the previous iteration finsihes above-- we could use it as it is------------
  
   //----------definition for patch level properties----- 
   
    //face normal vector
    vectorField nHat = this->patch().nf(); 
    //Info << "nHat" << nHat; 
    // cell centre value
    vectorField pif = this->patchInternalField();
    // gradient at the patch level 
    vectorField gradient = this->snGrad();
    //Info << "grdient" << gradient[50] << "g.x"<<gradient[50].component(0) << "g.y" <<gradient[50].component(1) <<"g.z" << gradient[50].component(2) << endl;
    //Info << "gradient =" << gradient[50] <<endl;
    vectorField gradient1 = this->snGrad(); //--- used for definition at the face level
    // only tangential components
    gradient = transform(I - sqr(nHat), gradient); 	
    // gradient Direction (since the pow function doesn't take vectors, we later have to multiply the magnitude of the gradient with the gradient direction)
     vectorField gradientDirection = gradient / (mag(gradient) + SMALL);  
    // obtain 1/del_y for the patch
     scalarField invdelypatch = this->patch().deltaCoeffs();
    //Info << "dely" <<invdelypatch<< endl;
    // defining the wall slip velocity in the previous iteration
     vectorField u_wallslip_prev = *this;
     vectorField u_wallslip = *this; // --- it is an initialization which would change over loop of faces
     //Info <<"IF" << pif[50] << "boun" << u_wallslip_prev[50] <<endl;
    const labelList cells = patch().faceCells();
    //Info<< "cellcenre"<<pif[50]<<"nuwface"<<nuw[50]<<"hfcell"<<invdelypatch[50];
    //-------------------------------looping over the facecells starts here-------------------------------------
     const fvMesh& mesh     = patch().boundaryMesh().mesh();
    const faceList& faces  = mesh.faces();
    const label& startFace = patch().patch().start();
    const label    endFace = startFace + size() - 1 ;
    label patchID = mesh.boundaryMesh().findPatchID("patchI");
    const polyPatch& cPatch = mesh.boundaryMesh()[patchI];
    label faceId_start = cPatch.start(); 
    forAll(cPatch,facei)   
    {
       //obtain the nuw for the face from the patch
       scalar nuw_face = nuw[facei];
       
       //getting the 1/del_y for the individual face
       scalar invdely = invdelypatch[facei];
           
       //gradient of velocity in cell in previous iteration
       vector gradient_prev_face = gradient1[facei];
       //obtain the normal for the face
       vector nHat_face = nHat[facei];
       //gradient of velocty in the prev iteration---- only tangential 
       gradient_prev_face = transform(I -sqr(nHat_face),gradient_prev_face);
       //initialise the guess limits for the bisection method
       scalar num = (ka1_*ka2_*nuw_face + (ka1_/invdely))/(ka2_*ka1_*nuw_face + (1/invdely));
       vector a (0,0,0);
       //vector b = pif[facei];
       vector b = num*pif[facei];
       //introduce the guess function
       vector guess_value = 0.5*(a+b);
       //Info << "gv" << guess_value<<endl;
       //defining the internal field of the cell of the face in the previous iteration
       vector pif_face = pif[facei];
       //defining the new gradient with the guess value and the internal field of the cell
       vector gradient_face = (guess_value - pif_face)*invdely;
       //obtaining only the tangential components of the gradient 
       gradient_face = transform(I -sqr(nHat_face),gradient_face);
       //obtaining the direction of the gradient
       vector gradient_cellDirection = gradient_face / (mag(gradient_face) + SMALL);
       //obtain the residual in the transcedent equation
       vector u_wallslip_func = guess_value - ka1_*(Foam::log(1-ka2_*nuw_face*mag(gradient_face)))*gradient_cellDirection; 
       //set the guess value as the root if not needed to go into the loop    
       u_wallslip[facei] = guess_value;
       // ---------start the boolean here to apply bisection methood -------------------
       scalar j = 0;
       scalar threshold = (b.component(0) - a.component(0))/(Foam::pow(2,j)); 
       while(threshold > 1e-9)
       {
       //introduce the guess function
        guess_value = 0.5*(a+b);
        
        //Info << "guess_value" << guess_value<<endl;
       //defining the internal field of the cell of the patch
        pif_face = pif[facei];
       //defining the new gradient with the guess value and the internal field of the cell
        gradient_face = (guess_value - pif_face)*invdely;
       //obtain the normal for the face
        nHat_face = nHat[facei];
       //obtaining only the tangential components of the gradient 
        gradient_face = transform(I -sqr(nHat_face),gradient_face);
        gradient_cellDirection = gradient_face / (mag(gradient_face) + SMALL);
       //Info << "direction" << gradient_cellDirection; 
        vector u_wallslip_func = guess_value - ka1_*(Foam::log(1-ka2_*nuw_face*mag(gradient_face)))*gradient_cellDirection;
       //obtain only the tangential components of the residual from the trascedent equation
        scalar tan_uws_func = u_wallslip_func.component(0);
       //apply the conditions of bosection- method to transcedent equation to find the root
       
        if (tan_uws_func  >0)
        {
            b = guess_value;
        }
        else if (tan_uws_func <0)
        {
            a = guess_value;
        }

        vector u_wallslip_face = guess_value;
        //tan_uws_func =mag(u_wallslip_cell.component(0));
        //Info << "tan_vel =" << tan_uws_func;
        u_wallslip[facei] = u_wallslip_face;
        j = j +1;
        //Info << "a="<< a <<"  ;  "<< "b=" << b<< endl;
        threshold = (b.component(0) -a.component(0))/(Foam::pow(2,j));
        //Info <<"threshold_loop" << threshold;
       }
 // i = i +1;
   //Info << "wallslip" << u_wallslip[facei];
  } 
    Info<< "u_walllslip= " <<u_wallslip[50]<<endl;      
    //Calculate and set u_wallslip       
    vectorField::operator=(relaxationFactor_*u_wallslip_prev+(1.0-relaxationFactor_)*u_wallslip);
    fixedValueFvPatchVectorField::updateCoeffs();    
}



// Write
void asymSlipGenNewtoniansemiFvPatchVectorField::write(Ostream& os) const
{
    fvPatchVectorField::write(os);
    os.writeKeyword("rho") << rho_ << token::END_STATEMENT << nl;
    os.writeKeyword("ka2") << ka2_ << token::END_STATEMENT << nl;
    os.writeKeyword("ka1") << ka1_ << token::END_STATEMENT << nl;
    os.writeKeyword("relaxationFactor") << relaxationFactor_ << token::END_STATEMENT << nl;  
    writeEntry("value", os);
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

makePatchTypeField(fvPatchVectorField, asymSlipGenNewtoniansemiFvPatchVectorField);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
