/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright held by original author
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#ifndef hatziSlipGenNewtonianRelaxationFvPatchVectorField_H
#define hatziSlipGenNewtonianRelaxationFvPatchVectorField_H

#include "fvPatchFields.H"
#include "fixedValueFvPatchFields.H"
#include "transformFvPatchField.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
              Class radiationConvectionFvPatchField Declaration
\*---------------------------------------------------------------------------*/

class hatziSlipGenNewtonianRelaxationFvPatchVectorField
:
    public fixedValueFvPatchVectorField
{
    // Private data

        //- fluid density
        scalar rho_;

        //- hatzikiriakos constant 2
        scalar kh2_;

        //- hatzikriakos constant 1
        scalar kh1_;

 	//- relaxation factor
        scalar relaxationFactor_;

public:

    //- Runtime type information
    TypeName("hatziSlipGenNewtonianRelaxation");


    // Constructors

        //- Construct from patch and internal field
        hatziSlipGenNewtonianRelaxationFvPatchVectorField
        (
            const fvPatch&,
            const DimensionedField<vector, volMesh>&
        );

        //- Construct from patch, internal field and dictionary
        hatziSlipGenNewtonianRelaxationFvPatchVectorField
        (
            const fvPatch&,
            const DimensionedField<vector, volMesh>&,
            const dictionary&
        );

        //- Construct by mapping given nonLinNavSlipGenNewtonianRelaxationFvPatchField
        //  onto a new patch
        hatziSlipGenNewtonianRelaxationFvPatchVectorField
        (
            const hatziSlipGenNewtonianRelaxationFvPatchVectorField&,
            const fvPatch&,
            const DimensionedField<vector, volMesh>&,
            const fvPatchFieldMapper&
        );

	//- Construct as copy
	hatziSlipGenNewtonianRelaxationFvPatchVectorField
	(
	    const hatziSlipGenNewtonianRelaxationFvPatchVectorField&
	);

        //- Construct and return a clone
        virtual tmp<fvPatchVectorField> clone() const
        {
            return tmp<fvPatchVectorField>
            (
                new hatziSlipGenNewtonianRelaxationFvPatchVectorField(*this)
            );
        }

        //- Construct as copy setting internal field reference
        hatziSlipGenNewtonianRelaxationFvPatchVectorField
        (
            const hatziSlipGenNewtonianRelaxationFvPatchVectorField&,
            const DimensionedField<vector, volMesh>&
        );

        //- Construct and return a clone setting internal field reference
        virtual tmp<fvPatchVectorField> clone
        (
            const DimensionedField<vector, volMesh>& iF
        ) const
        {
            return tmp<fvPatchVectorField>
            (
                new hatziSlipGenNewtonianRelaxationFvPatchVectorField(*this, iF)
            );
        }


    // Member functions


        //- Return the fluid density
        const scalar& rho() const
        {
            return rho_;
        }

        //- Return hatzikiriakos constant1 
        const scalar& kh1() const
        {
            return kh1_;
        }

        //- Return hatzikiriakos constant 1
        const scalar& kh2() const
        {
            return kh2_;
        }

        //- Return relaxationFactor

        const scalar& relaxationFactor() const
        {
            return relaxationFactor_;
        }

        //- Return non-const acess to the fluid density
        scalar& rho()
        {
            return rho_;
        }

        //- Return non-constant access to hatzikiriakos constant 1
        scalar& kh1()
        {
            return kh1_;
        }

        //- Return non-const acess to the n value
        scalar& kh2()
        {
            return kh2_;
        }

        //- Return non-const acess to the relaxationFactor value

        scalar& relaxationFactor()
        {
            return relaxationFactor_;
        }

        //- Update coefficients
        virtual void updateCoeffs();

        //- Write
        virtual void write(Ostream&) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
